Lib.Generators = (function() {
    var obj = {};
    var repo = Lib.createRepo();

    //generates linear travels objectst - clouds, birds, etc
    function LinearTraveler_Gen(opts) {
        var self = this;
        var opts = opts || {};
        var maxY = Lib.check.isNumber(opts.maxY) ? opts.maxY : game.world.height;
        var minY = Lib.check.isNumber(opts.minY) ? opts.minY : 0;
        var leftX = Lib.check.isNumber(opts.leftX) ? opts.leftX : -300;
        var rightX = Lib.check.isNumber(opts.rightX) ? opts.rightX : game.world.width + 300;
        var texture = Lib.check.isString(opts.texture) || Lib.check.totalArrCheck(opts.texture, "string") ? opts.texture : -1;
        if (Lib.check.isNumber(texture)) {
            Lib.glob.warn("Cant create Linear Travel Generator. No texture arg passed or texture wasn't loaded");
            return -1;
        }

        var group = Lib.check.isObject(opts.group) || Lib.check.totalArrCheck(opts.group, "object") ? opts.group: -1;

        var maxCount = Lib.check.isNumber(opts.maxCount) ? opts.maxCount : 5;
        var minRandTime = Lib.check.isNumber(opts.minRandTime) ? opts.minRandTime : 3;
        var maxRandTime = Lib.check.isNumber(opts.maxRandTime) ? opts.maxRandTime : 3;
        //The smallest ratio will be tested against a happens check (UTILS.happens) - if yes then it will be generated from the respective x(left or right)
        //If not then it will be generated on the oposite x (left or right)
        //The bigger ratio has no effect on the generation, only the smallest. Set the biggest to 1 for good practice
        var genLeftRatio = Lib.check.isNumber(opts.genLeftRatio) ? opts.genLeftRatio : 0.5;
        var genRightRatio = Lib.check.isNumber(opts.genRightRatio) ? opts.genRightRatio : 0.5;
        //Speed will be converted to positive values, generator determines if left or right
        var minSpeed = Lib.check.isNumber(opts.minSpeed) ? Math.abs(opts.minSpeed) : 100;
        var maxSpeed = Lib.check.isNumber(opts.maxSpeed) ? Math.abs(opts.maxSpeed) : 200;
        //scale.
        var minScale = Lib.check.isNumber(opts.minScale) ? opts.minScale : 0.5;
        var maxScale = Lib.check.isNumber(opts.maxScale) ? opts.maxScale : 1;
        //alpha
        var minAlpha = Lib.check.isNumber(opts.minAlpha) ? opts.minAlpha : 0.5;
        var maxAlpha = Lib.check.isNumber(opts.maxAlpha) ? opts.maxAlpha : 1;
        //
        var forwardFaceDirection = Lib.check.isString(opts.forwardFaceDirection) ? opts.forwardFaceDirection : "left";
        //anim speed min max
        var minAnimSpeed = Lib.check.isNumber(opts.minAnimSpeed) ? opts.minAnimSpeed : 0;
        var maxAnimSpeed = Lib.check.isNumber(opts.maxAnimSpeed) ? opts.maxAnimSpeed : 0;
        //if start count bigger then 0 it will generate startCount items on screen. Won't be bigger then max count
        var startOnScreenCount = Lib.check.isNumber(opts.startOnScreenCount) ? opts.startOnScreenCount : 0;
        //Lineal traveler specific args #################################################################################
        var minRandTimeChangeY = Lib.check.isNumber(opts.minRandTimeChangeY) ? opts.minRandTimeChangeY: 3;
        var maxRandTimeChangeY = Lib.check.isNumber(opts.maxRandTimeChangeY) ? opts.maxRandTimeChangeY: 6;
        var minYSpeed = Lib.check.isNumber(opts.minYSpeed) ? opts.minYSpeed : 0;
        var maxYSpeed = Lib.check.isNumber(opts.maxYSpeed) ? opts.maxYSpeed : 0;
        //if false it will start changing y after minRandTimeY. THis means teh object will tavel only on x, no change on y and then 
        //when minRandTimeChangeY passes it will start changing. 
        //Setting it to true will start changing Y as soon as it's born
        //false changeYAtStart it's good for when you want the object to enter horiz and after a time go up, down or in zigzag
        var changeYAtStart = Lib.check.isBoolean(opts.changeYAtStart) ? opts.changeYAtStart: true;
        //if to check for min and max y. 
        var checkForYLimits = Lib.check.isBoolean(opts.checkForYLimits) ? opts.checkForYLimits : true;


        //PRIVATE####################
        var _currentCount=0;


        function generate(onScreen=false){
            if(_currentCount>=maxCount){
                return 0;
            }
            // p("Generating Cloud");
            var y = Lib.UTILS.randInt(minY,maxY);
            var x=0;
            var speed =0;
            // var x = _getX();
            if(onScreen){
                x = _getOnScreenX();
                speed = _getOnScreenSpeed();
            }
            else{
                x = _getX();
                speed = _getSpeed(x);
            }
            // var speed = _getSpeed(x);
            var tex = _getTexture();
            var randGroup = _getGroup();
            var scale = Lib.UTILS.randFloat(minScale,maxScale);
            var alpha = Lib.UTILS.randFloat(minAlpha,maxAlpha);

            var linearTraveler = new Lib.FxObjects.LinearTraveler({
                x:x,
                y:y,
                speed:speed,
                texture:tex,
                group:randGroup,
                minRandTimeChangeY:minRandTimeChangeY,
                maxRandTimeChangeY:maxRandTimeChangeY,
                minYSpeed:minYSpeed,
                maxYSpeed:maxYSpeed,
                changeYAtStart:changeYAtStart,
                minY: minY,
                maxY:maxY,
                checkForYLimits: checkForYLimits
            });
            //post creation adjustments
            linearTraveler.obj.scale.x = scale;
            linearTraveler.obj.scale.y = scale;
            linearTraveler.obj.alpha = alpha;
            //flip horizontally if needed base on original face direction
            _flipTowardsDirection(speed,linearTraveler.obj,forwardFaceDirection);
            //add a random anim speed
            linearTraveler.anim.speed = Lib.UTILS.randInt(minAnimSpeed,maxAnimSpeed);
            // linearTraveler.anim.speed = 1;


            linearTraveler.onDestroy.add(_onDestroyCallback,this,[34]);
            _currentCount++;

            // p("Current Count is " + _currentCount);
        }

        function _onDestroyCallback (x){
            _currentCount--;
            if(_currentCount<0){
                _currentCount=0;
            }
        }

        //flips an obj to travel towards destination
        function _flipTowardsDirection(speed,obj,forwardFaceDirection){
            if(speed > 0){
                //it goes to right
                if(forwardFaceDirection === "left"){
                    obj.scale.x*=-1;
                }
            }

            else if(speed <=0){
                //it goes to left
                if(forwardFaceDirection === "right"){
                    obj.scale.x*=-1;
                }
            }

        }

        function _getTexture(){
            if(!Lib.check.isArray(texture)){
                return texture;
            }
            else{
                tex = Lib.UTILS.getRandValFromArr(texture);
                // p("Texture rand " + tex);
                return tex;
            }

        }

        function _getGroup(){
            if(!Lib.check.isArray(group)){
                return group;
            }
            else{
                gr = Lib.UTILS.getRandValFromArr(group);
                // p("Group " + gr);
                return gr;
            }
        }

        function _getSpeed(x){
            var speed = Lib.UTILS.randFloat(minSpeed,maxSpeed);
            //go towards left if x bigger than 0, otherwise leave it as it is
            if(x > 0){
                speed=-speed;
            }
            // p("Sped is " + speed);
            return speed;
        }

        function _getX(){
            //left or right
            var x=0;

            if(genRightRatio <= genLeftRatio){
                // p("right is smaller then left");
                if(Lib.UTILS.happens(genRightRatio)){
                    x = rightX;
                }
                else{
                    x = leftX;
                }
            }
            else if(genLeftRatio< genRightRatio){
                // p("right is bigger then left");
                if(Lib.UTILS.happens(genLeftRatio)){
                    x = leftX;
                }
                else{
                    x = rightX;
                }
            }

            else {
                // p(" otherwise ");
                x = leftX;
            }
            return x;
        }

        //gets an x coord on screen
        function _getOnScreenX(){
            var x=0;
            function _inScreenX(){
                return Lib.UTILS.randInt(0,game.world.width);
            }
            return _inScreenX();
        }

        //random speeds based on left or right ratio
        function _getOnScreenSpeed(){
            var speed = Lib.UTILS.randFloat(minSpeed,maxSpeed);

            if(genRightRatio <= genLeftRatio){
                if(Lib.UTILS.happens(genRightRatio)){
                    speed =  -speed;
                }
            }           
            else if(genLeftRatio < genRightRatio){
                if(Lib.UTILS.happens(genLeftRatio)){
                    speed = speed;
                }
            }
            return speed;

        }

        function _startGenerateOnScreen(){
            for(var a =0; a< startOnScreenCount;a++){
                //generate on screen
                generate(true);
            }
        }

        var eventRandTime = Lib.UTILS.randFloat(minRandTime, maxRandTime);
        var event = game.time.events.add(Phaser.Timer.SECOND * eventRandTime, nextEvent, this);

        function nextEvent() {
            randTime = Lib.UTILS.randFloat(minRandTime, maxRandTime);
            // this.();
            generate();
            event = game.time.events.add(Phaser.Timer.SECOND * eventRandTime, nextEvent, this);

        }


        //Called at end
        _startGenerateOnScreen();


    }

    obj.LinearTraveler_Gen = LinearTraveler_Gen

    //---
    return obj;
}());
